#!/usr/bin/env tclsh
# -*- tcl -*-
# Run an entire tournament of bots in a game world.
# Arguments
# - Directory holding the player bots
# - Directory holding the game files (including RunGame.tcl)

package require Tcl 8.6
package require fileutil

proc main {} {
    cmdline
    gameon
}

proc cmdline {} {
    global argv world players
    if {[llength $argv] != 2} usage
    lassign $argv world players
    set world   [file normalize $world]
    set players [file normalize $players]
    return
}

proc usage {} {
    global argv0
    puts stderr "Usage: $argv0 worlddir playerdir"
    exit 1
}

proc stop {args} {
    global argv0
    puts stderr "$argv0 error: $args"
    exit 1
}

proc gameon {} {
    global world players

    set players [glob -directory $players *.tcl]
    if {![llength $players]} {stop no players found}

    initranks $players

    cd $world
    file delete -force logs
    file mkdir         logs

    set counter 1
    foreach match [matches $players] {
	play $counter $match
	incr counter
    }

    showranks
    return
}

proc initranks {players} {
    global points
    foreach p $players {
	set np [file rootname [file tail $p]]
	dict set points $np 0
    }
    return
}

proc showranks {} {
    global points

    # invert
    dict for {player points} $points {
	dict lappend rank $points $player
    }

    array set ranking $rank
    parray ranking
    return
}

proc record {pa pb winner} {
    global points
    if {$winner eq "tie"} {
	dict incr points $pa 1
	dict incr points $pb 1
    } elseif {$winner eq $pa} {
	dict incr points $pa 2
    } elseif {$winner eq $pb} {
	dict incr points $pb 2
    } else {
	stop Bad winner <$winner>
    }
    return
}

proc play {game match} {
    lassign $match a b
    set na [file rootname [file tail $a]]
    set nb [file rootname [file tail $b]]

    puts -nonewline "\[[format %3d $game]\] Playing: \"$na\" vs \"$nb\" ... "
    flush stdout

    set label game$game
    set log logs/${label}.log

    # Note: We ignore the low-level log.
    # This code assumes the modified RunGame.tcl with the higher-level log in stderr.
    exec 2> $log > ${log}.p \
	[info nameofexecutable] RunGame.tcl $label . WorldMap.tcl \
	$na $a \
	$nb $b

    # The winner of the game (ortie) is in the 2nd to last line.
    set result [string trim [lindex [split [fileutil::cat $log] \n] end-2]]
    puts $result

    if {$result eq "tie"} {
	record $na $nb tie
    } else {
	# Line = ('winner' $turns $name)
	record $na $nb [lindex $result 2]
    }
}

proc matches {players} {
    # generate the matchups to play

    # Rule
    # - everybody against everybody else, except themselves.
    #   - This automatically gives each combination a/b to games, namely a/b and b/a.

    set matches {}
    foreach a $players {
	foreach b $players {
	    if {$a eq $b} continue
	    lappend matches [list $a $b]
	}
    }
    return $matches
}

main
